@@ -29,7 +29,7 @@ class TestCaseNotVulnerable(TlsSuiteTester):
         server_profile = tlsmate.server_profile
         ScanCcsInjection(tlsmate).run()
         profile = server_profile.make_serializable()
-        assert profile["vulnerabilities"]["ccs_injection"] == "C_FALSE"
+        assert profile["vulnerabilities"]["ccs_injection"] == "FALSE"
 
 
 if __name__ == "__main__":
