@@ -4,17 +4,18 @@
 # import basic stuff
 
 # import own stuff
-from tlsmate.plugin import WorkerPlugin
+from tlsmate.plugin import Worker
 from tlsmate import utils
 
 # import other stuff
 
 
-class ReadProfileWorker(WorkerPlugin):
-    """WorkerPlugin class which deserializes a server profile in Yaml format
+class ReadProfileWorker(Worker):
+    """Worker class which deserializes a server profile in Yaml format
     """
 
     name = "profile_reader"
+    descr = "read the server profile"
     prio = 1
 
     def run(self):
@@ -23,11 +24,12 @@ class ReadProfileWorker(WorkerPlugin):
             self.server_profile.load(utils.deserialize_data(read_profile))
 
 
-class DumpProfileWorker(WorkerPlugin):
-    """WorkerPlugin class which serializes a server profile.
+class DumpProfileWorker(Worker):
+    """Worker class which serializes a server profile.
     """
 
     name = "profile_dumper"
+    descr = "dump the server profile"
     prio = 1001
 
     def run(self):
